<?php
/**
 * Created by PhpStorm.
 * User: Skalden
 * Date: 6/23/2018
 * Time: 1:54 PM
 */

namespace App\Http\Controllers;

use App\Models\Address;
use App\Models\Images;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Hash;
use Exception;

class ProfileController extends Controller
{
    public function info()
    {
        return view('pages.profile.user', [
            'user' => User::find(Auth::id()),
            'addresses' => Address::where('user_id', Auth::id())->get(),
        ]);
    }

    /**
     * Добавление адреса
     *
     * @param Request $request
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function address_add(Request $request)
    {
        $address = new Address();

        if ($request->isMethod('post')) {
            try {
                $this->handle_save($address, $request);
                Session::flash('success', 'Адресс успешно добавлен');
                return redirect('/profile/address/edit/' . $address->id);
            } catch (Exception $ex) {
                Session::flash('error', 'Не удалось сохранить данные: ' . $ex->getMessage());
            }
        }

        return view('pages.profile.addresses_edit', [
            'address' => $address
        ]);
    }

    /**
     * Редактирование адреса
     *
     * @param Request $request
     * @param $id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function address_edit(Request $request, $id)
    {
        $address = Address::find((int)$id);

        if ($request->isMethod('post')) {
            try {
                $this->handle_save($address, $request);
                Session::flash('success', 'Адресс успешно обновлен');
            } catch (Exception $ex) {
                Session::flash('error', 'Не удалось сохранить данные: ' . $ex->getMessage());
            }
        }

        return view('pages.profile.addresses_edit', [
            'address' => $address
        ]);
    }

    /**
     * Удаление адреса
     *
     * @param $id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function address_delete($id)
    {
        $address = Address::find((int)$id);
        if ($address->exists) {
            $image = Images::find($address->scan_id);
            if ($image)
                $image->delete();
            $address->delete();
        }
        Session::flash('success', 'Адрес успешно удален');
        return redirect()->back();
    }

    /**
     * Установка адреса по умолчанию
     *
     * @param $id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function address_set_default($id)
    {
        $address = Address::where('user_id', Auth::id())->where('is_default', 1)->first();
        if ($address !== null) {
            $address->is_default = 0;
            $address->save();
        }

        $address = Address::find((int)$id);
        if ($address !== null) {
            $address->is_default = 1;
            $address->save();
        }

        Session::flash('success', 'Установлен новый адрес для доставки по умолчанию');
        return redirect()->back();

    }

    /**
     * Редактирование профиля
     *
     * @param Request $request
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function edit_profile(Request $request)
    {
        $model = User::find(Auth::id());

        if ($request->isMethod('POST')) {
            try {
                $this->handle_profile_save($model, $request);
                Session::flash('success', 'Профиль успешно обновлен');
            } catch (Exception $ex) {
                Session::flash('error', 'Ошибка сохранения профиля: ' . $ex->getMessage());
            }
        }
        return view('pages.profile.profile_edit', [
            'model' => $model
        ]);
    }

    /**
     * Функция изменения пароля
     *
     * @param Request $request
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function edit_password(Request $request)
    {
        $model = User::find(Auth::id());

        if ($request->isMethod('POST')) {
            try {
                $this->handle_password_change($model, $request);
                Session::flash('success', 'Пароль успешно обновлен');
            } catch (Exception $ex) {
                Session::flash('error', 'Ошибка сохранения пароля: ' . $ex->getMessage());
            }
        }
        return view('pages.profile.change_password', [
            'model' => $model
        ]);
    }

    /**
     * Обоработка измненения пароля
     *
     * @param $model
     * @param $request
     * @throws Exception
     */
    private function handle_password_change(&$model, &$request)
    {
        if ($request->input('password') != $request->input('passwordConfirmation'))
            throw new Exception('Поля пароль и подтверждени пароля не совпадают');

        $model->password = Hash::make($request->input('password'));
        $model->ip = $request->ip();
        if (!$model->save())
            throw new Exception('error');
    }

    /**
     * Функция оброботки сохранения
     *
     * @param $model
     * @param $request
     * @throws Exception
     */
    public function handle_profile_save(&$model, &$request)
    {
        $model->name = $request->input('name');
        $model->email = $request->input('email');
        $model->firstname = $request->input('firstname');
        $model->lastname = $request->input('lastname');
        $model->phone = $request->input('phone');
        $model->ip = $request->ip();

        if (!$model->save())
            throw new Exception('error');
    }

    /**
     * Сохранение информации об адресе
     *
     * @param $request
     * @throws Exception
     */
    private function handle_save(&$address, $request)
    {
        $address->user_id = Auth::id();
        $address->firstname = $request->input('firstname');
        $address->secondname = $request->input('secondname');
        $address->fathername = $request->input('fathername');
        $address->phone = $request->input('phone');
        $address->country = $request->input('country');
        $address->oblast = $request->input('oblast');
        $address->city = $request->input('city');
        $address->street = $request->input('street');
        $address->house = $request->input('house');
        $address->apartment = $request->input('apartment');
        $address->zip = $request->input('zip');
        $address->created = date('Y-m-d H:i:s');

        if ($request->hasFile('scanFile')) {
            if (!$request->file('scanFile')->isValid())
                throw new Exception('Ошибка валидации файла');

            $file = $request->file('scanFile');
            $path = 'uploads';
            $path .= '/' . date('Ymd');
            $filename = md5(time()) . date('Ymdhis') . '.' . $file->getClientOriginalExtension();
            $file->move($path, $filename);

            $image = new Images();
            $image->name = $filename;
            $image->upload_name = $file->getClientOriginalName();
            $image->path = $path;
            $image->size = $file->getSize();
            $image->date = date('Y-m-d H:i:s');

            if (!$image->save())
                throw new Exception('Не удается сохранить информацию о скан-копии паспорта');

            $address->scan_id = $image->id;
        }

        if (!$address->save())
            throw new Exception('Невозможно сохранить модель');
    }

    public function address()
    {
        $units = Address::where('user_id', '=', Auth::user()->id)->get();
        return view('pages.profile.addresses', compact('units'));
    }

    public function addaddress()
    {
        return view('pages.profile.address_add');
    }

    public function storeaddress(Request $request)
    {

        $address = new Address;
        $address->fill($request->all());
        $address->user_id = Auth::user()->id;
        $address->save();

        return redirect()->route('address');
    }

    public function destroyaddress($id)
    {
        Address::destroy($id);

        return redirect()->back()->with('success', 'Адрес удален');
    }

    public function profile()
    {

        return view('pages.profile.user');
    }

    /**
     * Save user's profile information
     *
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function save(Request $request)
    {
        try {
            $user = User::find(Auth::user()->id);

            $user->name = $request->input('name');
            $user->surname = $request->input('surname');
            $user->email = $request->input('email');
            $user->phone = $request->input('phone');

            if ($request->filled('password') && $request->filled('password_confirmation')) {
                $user->password = Hash::make($request->input('password'));
            }

            $user->update();
        } catch (Exception $ex) {
            return redirect()->back()->with('error', $ex->getMessage());
        }

        return redirect()->back()->with('success', 'Информация успешно сохранена.');
    }
}
